pub struct OpStack {
pub stack: Vec<BFieldElement>,
/* private fields */
}Expand description
The operational stack of Triton VM.
It always contains at least OpStackElement::COUNT elements. Initially,
the bottom-most Digest::LEN elements equal the digest of the program
being executed. The remaining elements are initially 0.
The OpStack is represented as one contiguous piece of memory, and Triton VM
uses it as such. For reasons of arithmetization, however, there is a
distinction between the op-stack registers and the op-stack underflow
memory. The op-stack registers are the first OpStackElement::COUNT
elements of the op-stack, and the op-stack underflow memory is the remaining
elements.
Fields
stack: Vec<BFieldElement>The underlying, actual stack. When manually accessing, be aware of
reversed indexing: while op_stack[0] is the top of the stack,
op_stack.stack[0] is the lowest element in the stack.
Implementations
Sourceimpl OpStack
impl OpStack
pub fn new(program_digest: Digest) -> Self
pub fn len(&self) -> usize
pub fn push(&mut self, element: BFieldElement)
pub fn pop(&mut self) -> Result<BFieldElement, OpStackError>
pub fn insert(&mut self, index: OpStackElement, element: BFieldElement)
pub fn remove(&mut self, index: OpStackElement) -> BFieldElement
pub fn start_recording_underflow_io_sequence(&mut self)
pub fn stop_recording_underflow_io_sequence(&mut self) -> Vec<UnderflowIO>
pub fn push_extension_field_element(&mut self, element: XFieldElement)
pub fn pop_extension_field_element( &mut self, ) -> Result<XFieldElement, OpStackError>
pub fn is_u32(&self, stack_element: OpStackElement) -> Result<(), OpStackError>
pub fn get_u32( &self, stack_element: OpStackElement, ) -> Result<u32, OpStackError>
pub fn pop_u32(&mut self) -> Result<u32, OpStackError>
pub fn pop_multiple<const N: usize>( &mut self, ) -> Result<[BFieldElement; N], OpStackError>
pub fn peek_at_top_extension_field_element(&self) -> XFieldElement
pub fn would_be_too_shallow(&self, stack_delta: i32) -> bool
Sourcepub fn pointer(&self) -> BFieldElement
pub fn pointer(&self) -> BFieldElement
The address of the next free address of the op-stack. Equivalent to the current length of the op-stack.